package cn.newfeifan.mall.module.distri.controller.admin.dailybill.vo;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;

@Schema(description = "管理后台 - 每日账单新增/修改 Request VO")
@Data
public class DailyBillSaveReqVO {

    @Schema(description = "账单编号", requiredMode = Schema.RequiredMode.REQUIRED, example = "27497")
    private Long id;

    @Schema(description = "金额，单位：分", requiredMode = Schema.RequiredMode.REQUIRED, example = "30486")
    @NotNull(message = "金额，单位：分不能为空")
    private Long price;

    @Schema(description = "确收金额，单位：分", requiredMode = Schema.RequiredMode.REQUIRED, example = "29815")
    @NotNull(message = "确收金额，单位：分不能为空")
    private Long receivedPrice;

    @Schema(description = "订单数，默认为0", requiredMode = Schema.RequiredMode.REQUIRED, example = "27164")
    @NotNull(message = "订单数，默认为0不能为空")
    private Long orderCount;

    @Schema(description = "店铺ID", example = "11867")
    private Long shopId;

    @Schema(description = "商户ID", example = "22961")
    private Long merchantId;

    @Schema(description = "订单ID列表，文本类型", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotEmpty(message = "订单ID列表，文本类型不能为空")
    private String orderIds;

    @Schema(description = "退款金额, 单位: 分", requiredMode = Schema.RequiredMode.REQUIRED, example = "11962")
    @NotNull(message = "退款金额, 单位: 分不能为空")
    private Long refundPrice;

    @Schema(description = "积分", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "积分不能为空")
    private Long integral;

    @Schema(description = "确收积分", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "确收积分不能为空")
    private Long receivedIntegral;

    @Schema(description = "退款积分", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "退款积分不能为空")
    private Long refundIntegral;

    @Schema(description = "订单所属日期", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "订单所属日期不能为空")
    private LocalDate orderCalcTime;

    @Schema(description = "消费分", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "消费分不能为空")
    private Long consumptionPoints;

    @Schema(description = "确收消费分", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "确收消费分不能为空")
    private Long receivedConsumptionPoints;

    @Schema(description = "退款消费分", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "退款消费分不能为空")
    private Long refundConsumptionPoints;
}