package cn.newfeifan.mall.module.distri.service.dailybill;

import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.module.distri.controller.admin.dailybill.vo.DailyBillPageReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.dailybill.vo.DailyBillRespVO;
import cn.newfeifan.mall.module.distri.controller.admin.dailybill.vo.DailyBillSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.dailybill.DailyBillDO;

import javax.validation.Valid;
import java.time.LocalDate;

/**
 * 每日账单 Service 接口
 *
 * @author 非繁人
 */
public interface DailyBillService {

    /**
     * 创建每日账单
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDailyBill(@Valid DailyBillSaveReqVO createReqVO);

    /**
     * 更新每日账单
     *
     * @param updateReqVO 更新信息
     */
    void updateDailyBill(@Valid DailyBillSaveReqVO updateReqVO);

    /**
     * 删除每日账单
     *
     * @param id 编号
     */
    void deleteDailyBill(Long id);

    /**
     * 获得每日账单
     *
     * @param id 编号
     * @return 每日账单
     */
    DailyBillDO getDailyBill(Long id);

    /**
     * 获得每日账单分页
     *
     * @param pageReqVO 分页查询
     * @return 每日账单分页
     */
    PageResult<DailyBillRespVO> getDailyBillPage(DailyBillPageReqVO pageReqVO);

    /**
     * 计算每家店铺每日的订单
     */
    void caleDailyBill();

    /**
     * 获取每日账单
     * @param merchantId 商户ID
     * @param shopId 店铺ID
     * @param localDate 日期
     * @return 每日账单
     */
    DailyBillDO getDailyBill(Long merchantId, Long shopId, LocalDate localDate);
}