package cn.newfeifan.mall.module.distri.service.integral;

import cn.newfeifan.mall.framework.common.util.object.BeanUtils;
import cn.newfeifan.mall.module.distri.controller.admin.consumptionchangelog.vo.ConsumptionChangeLogSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.integral.vo.IntegralSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.ptprofitlog.vo.PtProfitLogSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.integral.IntegralDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.orderpercentage.OrderPercentageDO;
import cn.newfeifan.mall.module.distri.dal.mysql.integral.IntegralMapper;
import cn.newfeifan.mall.module.distri.dal.mysql.ptprofit.PtProfitMapper;
import cn.newfeifan.mall.module.distri.enums.CaclEnum;
import cn.newfeifan.mall.module.distri.enums.ConsumptionEnum;
import cn.newfeifan.mall.module.distri.service.consumptionchangelog.ConsumptionChangeLogService;
import cn.newfeifan.mall.module.distri.service.orderpercentage.OrderPercentageService;
import cn.newfeifan.mall.module.distri.service.ptprofitlog.PtProfitLogService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Map;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.INTEGRAL_NOT_EXISTS;

/**
 * 推荐用户积分 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class IntegralServiceImpl implements IntegralService {

    @Resource
    private PtProfitMapper ptProfitMapper;
    @Resource
    private IntegralMapper integralMapper;
    @Resource
    private PtProfitLogService ptProfitLogService;
    @Resource
    private OrderPercentageService orderPercentageService;

    @Resource
    private ConsumptionChangeLogService consumptionChangeLogService;

    /**
     * add by Ben
     * 修改用户积分
     *
     * @param userId       要修改积分的用户
     * @param caclEnum     导致积分变化的事件类型枚举
     * @param amount       用户可用积分变化金额，可为负数
     * @param freezeAmount 用户冻结积分变化金额，可为负数
     */
    @Override
    public Boolean updateUserIntegral(Long generateUserId,//导致产生积分变动的用户ID
                                   Long userId, CaclEnum caclEnum, Long amount, Long freezeAmount,
                                   Long tradeOrderId,
                                   String orderNum
    ) {
        Boolean success = null;

        //修改用户积分钱包的积分
        int effectRowNum = ptProfitMapper.updateUserIntegral(amount,amount,freezeAmount,freezeAmount,userId);

        if (effectRowNum==1) {//如果成功修改积分记录
            success = true;

            //用户修改后的当前可用的钱包余额
            Long afterAmount = null;
            //用户修改后的冻结积分钱包余额
            Long afterFreezeAmount = null;

            //查询用户修改后的积分
            Map<String, Object> map = ptProfitMapper.selectUserIntegral(userId);
            Object currentQuotaObj = map.get("current_quota");
            Object freezeQuotaObj = map.get("freeze_quota");

            afterAmount = (Long)currentQuotaObj;
            afterFreezeAmount = (Long)freezeQuotaObj;

            if(afterAmount<0||afterFreezeAmount<0){
                throw new RuntimeException("用户钱包为负数：用户IO为"+userId+"的用户可用钱包为"+afterAmount+",冻结钱包为"+afterFreezeAmount);
            }

            //记录用户的积分变动日志
            ptProfitLogService.addMessage(generateUserId,//导致产生积分变动的用户ID
                    userId, caclEnum, amount, afterAmount,
                    freezeAmount, afterFreezeAmount,null,tradeOrderId,orderNum);

        }else if(effectRowNum<1){//如果钱包积分不足，不够抵扣
            success = false;
        } else if (effectRowNum>1) {
//            success = false;
            throw new RuntimeException("用户IO为"+userId+"的用户钱包不只一个！");
        }

        return success;

    }

    @Override
    public void updateUserIntegral(Long userId, CaclEnum caclEnum, Long payIntegral, Long tradeOrderId, String orderNum) {
        //退还订单支付过期积分
        IntegralDO integralDO = integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>().eq(IntegralDO::getUserId, userId));
        integralDO.setCurrentQuota(integralDO.getCurrentQuota() + payIntegral);

        integralMapper.updateById(integralDO);

        ptProfitLogService.addMessage(userId,userId,caclEnum,payIntegral,integralDO.getCurrentQuota(),0L,0L,null,tradeOrderId,orderNum);
    }

    @Override
    public void updateIntegralFreezeHighQuota(Long generateUserId, Long userId, CaclEnum caclEnum, Long totalFreezeHighQuota, Long tradeOrderId, String orderNum) {
        IntegralDO integralDO = integralMapper.selectOne(IntegralDO::getUserId, userId);
//        integralDO.setFreezeHighQuota(integralDO.getFreezeHighQuota() - totalFreezeHighQuota);
        integralDO.setHighQuota(integralDO.getHighQuota() - totalFreezeHighQuota);
        integralDO.setHighQuotaTotal(integralDO.getHighQuotaTotal() - totalFreezeHighQuota);
        integralMapper.updateById(integralDO);

        OrderPercentageDO orderPercentageDO = orderPercentageService.queryStatus();

        //记录用户的积分变动日志
        PtProfitLogSaveReqVO profitLogSaveReqVO = PtProfitLogSaveReqVO.builder()
//                .freezeHighQuota(-totalFreezeHighQuota)
//                .afterFreezeHighQuota(integralDO.getFreezeHighQuota())
                .maxAvailablePointsAmount(-totalFreezeHighQuota)
                .afterMaxAvailablePointsAmount(integralDO.getHighQuota() + Long.parseLong(orderPercentageDO.getBaseMaxQuota()))     // 系统设置的最大可用积分
                .userId(userId)
                .generateUserId(generateUserId)
                .profitStatus(caclEnum.getType())
                .orderId(tradeOrderId)
                .orderNo(orderNum)
                .build();
        ptProfitLogService.createPtProfitLog(profitLogSaveReqVO);
    }

    @Override
    public void updateUserConsumptionPoints(Long generateUserId, Long userId, Long payConsumptionPoints, Long tradeOrderId, String tradeOrderNum) {
        // 获取用户钱包,然后退回消费分
        IntegralDO integralDO = integralMapper.selectOne(IntegralDO::getUserId, userId);
        integralDO.setConsumptionPoints(integralDO.getConsumptionPoints() + payConsumptionPoints);
        integralMapper.updateById(integralDO);

        // 记录消费分变动日志
        consumptionChangeLogService.createConsumptionChangeLog(ConsumptionChangeLogSaveReqVO.builder()
                        .userId(userId)
                        .generateUserId(generateUserId)
                        .consumptionPoints(payConsumptionPoints)
                        .afterConsumptionPoints(integralDO.getConsumptionPoints())
                        .consumptionStatus(ConsumptionEnum.REFUND_ORDER.getType())
                .build());

    }

    @Override
    public void updateUserConsumptionPoints(Long userId, ConsumptionEnum consumptionEnum, Long consumptionPoints, Long orderId, String orderNum) {
        IntegralDO integralDO = integralMapper.selectOne(IntegralDO::getUserId, userId);

        // 修改用户消费积分
        integralDO.setConsumptionPoints(integralDO.getConsumptionPoints() + consumptionPoints);
        integralMapper.updateById(integralDO);

        consumptionChangeLogService.createConsumptionChangeLog(
                ConsumptionChangeLogSaveReqVO.builder()
                        .userId(userId)
                        .generateUserId(userId)
                        .consumptionStatus(consumptionEnum.getType())
                        .consumptionPoints(consumptionPoints)
                        .afterConsumptionPoints(integralDO.getConsumptionPoints())
                        .orderId(orderId)
                        .orderNo(orderNum)
                        .build());
    }

    @Override
    public IntegralDO selectByUser(Long descendant) {
        return integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>().eq(IntegralDO::getUserId, descendant));
    }

    @Override
    public void updateIntegral(IntegralSaveReqVO updateReqVO) {
        // 校验存在
        validateIntegralExists(updateReqVO.getId());
        // 更新
        IntegralDO updateObj = BeanUtils.toBean(updateReqVO, IntegralDO.class);
        integralMapper.updateById(updateObj);
    }

    @Override
    public void updateUserIntegralByAmountWithHighQuota(Long orderUserId, Long ancesterUserId, CaclEnum caclEnum, Long ancestorIncreaseIntegral, Long tradeOrderId, String tradeOrderNum) {
        IntegralDO integralDO = integralMapper.selectOne(IntegralDO::getUserId, ancesterUserId);
        integralDO.setCurrentQuota(integralDO.getCurrentQuota() - ancestorIncreaseIntegral);
        integralDO.setHighQuota(integralDO.getHighQuota() + ancestorIncreaseIntegral);
        integralMapper.updateById(integralDO);

        OrderPercentageDO orderPercentageDO = orderPercentageService.queryStatus();

        //记录用户的积分变动日志
        PtProfitLogSaveReqVO profitLogSaveReqVO = PtProfitLogSaveReqVO.builder()
                .amount(-ancestorIncreaseIntegral)
                .afterAmount(integralDO.getCurrentQuota())
                .maxAvailablePointsAmount(ancestorIncreaseIntegral)
                .afterMaxAvailablePointsAmount(integralDO.getHighQuota() + Long.parseLong(orderPercentageDO.getBaseMaxQuota()))     // 系统设置的最大可用积分
                .userId(ancesterUserId)
                .generateUserId(orderUserId)
                .profitStatus(caclEnum.getType())
                .orderId(tradeOrderId)
                .orderNo(tradeOrderNum)
                .build();
        ptProfitLogService.createPtProfitLog(profitLogSaveReqVO);
    }

    private void validateIntegralExists(Long id) {
        if (integralMapper.selectById(id) == null) {
            throw exception(INTEGRAL_NOT_EXISTS);
        }
    }

//
//    /**
//     * 修改用户身价
//     * @param userId 用户ID
//     * @param socialStatusUpdateEnum 身价变动类别
//     * @param socialStatusPoint 身价变化值
//     * @param ObjectId 导致身价变动的对应业务对象ID，如订单ID、收藏记录ID、新注册用户ID等
//     * @return 升级后的身价名称，如果返回null，表明没升级
//     */
//    public String updateUserSocialStatus(Long userId, SocialStatusUpdateEnum socialStatusUpdateEnum, Integer socialStatusPoint,
//                                   Long ObjectId){
//        //升级后的身价名称，如果返回null，表明没升级
//        String newSocialStatusName = null;
//        //用户身价升了几级
//        Integer socialStatusUpgradation = 0;
//
//        DuserDO duser = duserMapper.selectOne("user_id", userId);
//        Integer oldSocialStatus = duser.getSocialStatus();//升级前身价等级
//
//        return newSocialStatusName;
//    }
}