package cn.newfeifan.mall.module.product.service.skuapply;

import java.util.*;
import javax.validation.*;
import cn.newfeifan.mall.module.product.controller.admin.skuapply.vo.*;
import cn.newfeifan.mall.module.product.dal.dataobject.skuapply.SkuApplyDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

/**
 * 商品sku申请 Service 接口
 *
 * @author 非繁人
 */
public interface SkuApplyService {

    /**
     * 创建商品sku申请
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSkuApply(@Valid SkuApplySaveReqVO createReqVO);

    /**
     * 删除商品sku申请
     *
     * @param id 编号
     */
    void deleteSkuApply(Long id);

    /**
     * 获得商品sku申请
     *
     * @param id 编号
     * @return 商品sku申请
     */
    SkuApplyDO getSkuApply(Long id);

    /**
     * 获得商品sku申请分页
     *
     * @param pageReqVO 分页查询
     * @return 商品sku申请分页
     */
    PageResult<SkuApplyDO> getSkuApplyPage(SkuApplyPageReqVO pageReqVO);
    /**
     * 批量创建 SKU
     *
     * @param spuId 商品 SPU 编号
     * @param list  SKU 对象集合
     */
    void createSkuList(Long spuId, List<SkuApplySaveReqVO> list);

    /**
     * 获得商品 SKU 申请 集合
     *
     * @param spuApplyId spu 申请 编号
     * @return 商品sku 申请 集合
     */
    List<SkuApplyDO> getSkuListBySpuId(Long spuApplyId);

    /**
     * 根据 SPUApply 编号，批量更新它的 SKU 信息
     *
     * @param spuApplyId SPU 编码
     * @param skus  SKU 的集合
     */
    void updateSkuList(Long spuApplyId, List<SkuApplySaveReqVO> skus);

    /**
     * 对 sku 的组合的属性等进行合法性校验
     *
     * @param list          sku组合的集合
     * @param highPrecision 是否高精度商品
     */
    void validateSkuList(List<SkuApplySaveReqVO> list, Boolean specType, Boolean highPrecision);
}