package cn.newfeifan.mall.module.trade.service.order;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.pojo.echarts.EChartsResult;
import cn.newfeifan.mall.module.distri.dal.mysql.ordersettlement.OrderSettlementMapper;
import cn.newfeifan.mall.module.member.api.user.MemberUserApi;
import cn.newfeifan.mall.module.member.api.user.dto.MemberUserRespDTO;
import cn.newfeifan.mall.module.pay.controller.admin.order.vo.PayOrderPageItemRespVO;
import cn.newfeifan.mall.module.product.dal.dataobject.spu.ProductSpuDO;
import cn.newfeifan.mall.module.product.service.spu.ProductSpuService;
import cn.newfeifan.mall.module.system.controller.admin.user.vo.user.UserShopDetailsVO;
import cn.newfeifan.mall.module.system.service.user.AdminUserService;
import cn.newfeifan.mall.module.trade.controller.admin.order.vo.TradeOrderPageItemRespVO;
import cn.newfeifan.mall.module.trade.controller.admin.order.vo.TradeOrderPageReqVO;
import cn.newfeifan.mall.module.trade.controller.admin.order.vo.TradeOrderSummaryRespVO;
import cn.newfeifan.mall.module.trade.controller.app.order.vo.AppTradeOrderPageReqVO;
import cn.newfeifan.mall.module.trade.dal.dataobject.delivery.DeliveryExpressDO;
import cn.newfeifan.mall.module.trade.dal.dataobject.order.TradeOrderDO;
import cn.newfeifan.mall.module.trade.dal.dataobject.order.TradeOrderItemDO;
import cn.newfeifan.mall.module.trade.dal.mysql.order.TradeOrderItemMapper;
import cn.newfeifan.mall.module.trade.dal.mysql.order.TradeOrderMapper;
import cn.newfeifan.mall.module.trade.dal.redis.RedisKeyConstants;
import cn.newfeifan.mall.module.trade.enums.BacklogConstants;
import cn.newfeifan.mall.module.trade.enums.order.TradeOrderRefundStatusEnum;
import cn.newfeifan.mall.module.trade.enums.order.TradeOrderStatusEnum;
import cn.newfeifan.mall.module.trade.framework.delivery.core.client.ExpressClientFactory;
import cn.newfeifan.mall.module.trade.framework.delivery.core.client.dto.ExpressTrackQueryReqDTO;
import cn.newfeifan.mall.module.trade.framework.delivery.core.client.dto.ExpressTrackRespDTO;
import cn.newfeifan.mall.module.trade.service.aftersale.AfterSaleService;
import cn.newfeifan.mall.module.trade.service.delivery.DeliveryExpressService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.framework.common.util.collection.CollectionUtils.convertSet;
import static cn.newfeifan.mall.module.trade.enums.ErrorCodeConstants.EXPRESS_NOT_EXISTS;
import static cn.newfeifan.mall.module.trade.enums.ErrorCodeConstants.ORDER_NOT_FOUND;

/**
 * 交易订单【读】 Service 实现类
 *
 * @author 非繁源码
 */
@Service
@Slf4j
public class TradeOrderQueryServiceImpl implements TradeOrderQueryService {

    @Resource
    private ExpressClientFactory expressClientFactory;

    @Resource
    private TradeOrderMapper tradeOrderMapper;
    @Resource
    private TradeOrderItemMapper tradeOrderItemMapper;

    @Resource
    private DeliveryExpressService deliveryExpressService;

    @Resource
    private MemberUserApi memberUserApi;

    @Resource
    private AdminUserService userService;

    @Resource
    @Lazy
    private AfterSaleService afterSaleService;

    @Resource
    private ProductSpuService productSpuService;

    @Resource
    private OrderSettlementMapper orderSettlementMapper;
    @Resource
    private AdminUserService adminUserService;


    // =================== Order ===================

    @Override
    public TradeOrderDO getOrder(Long id) {
        return tradeOrderMapper.selectById(id);
    }

    @Override
    public TradeOrderDO getOrder(Long userId, Long id) {
        TradeOrderDO order = tradeOrderMapper.selectById(id);
        if (order != null && ObjectUtil.notEqual(order.getUserId(), userId)) {
            return null;
        }
        return order;
    }

    @Override
    public TradeOrderDO getOrderByUserIdAndStatusAndCombination(Long userId, Long combinationActivityId, Integer status) {
        return tradeOrderMapper.selectByUserIdAndCombinationActivityIdAndStatus(userId, combinationActivityId, status);
    }

    @Override
    public List<TradeOrderDO> getOrderList(Collection<Long> ids) {
        if (CollUtil.isEmpty(ids)) {
            return Collections.emptyList();
        }
        return tradeOrderMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<TradeOrderDO> getOrderPage(TradeOrderPageReqVO reqVO) {
        // 根据用户查询条件构建用户编号列表
        Set<Long> userIds = buildQueryConditionUserIds(reqVO);
        if (userIds == null) { // 没查询到用户，说明肯定也没他的订单
            return PageResult.empty();
        }

        UserShopDetailsVO userShopDetails = userService.getUserShopDetails();

        reqVO.setMerchantId(userShopDetails.getMerId());
        reqVO.setShopId(userShopDetails.getShopId());

        List<Long> orderIds = null;
        if (reqVO.getShopSettlementId() != null) {
            orderIds = orderSettlementMapper.selectOrderIdsByShopSettlementId(reqVO.getShopSettlementId());
        }

        // 分页查询
        PageResult<TradeOrderDO> tradeOrderDOPageResult = tradeOrderMapper.selectPage(reqVO, userIds,orderIds);
        if (reqVO.getStatus() != null && reqVO.getStatus().equals(TradeOrderStatusEnum.AFTER_SALE.getStatus())) {
            for (TradeOrderDO tradeOrderDO : tradeOrderDOPageResult.getList()) {
                tradeOrderDO.setStatus(TradeOrderStatusEnum.AFTER_SALE.getStatus());
            }
        }
        return tradeOrderDOPageResult;
    }

    private Set<Long> buildQueryConditionUserIds(TradeOrderPageReqVO reqVO) {
        // 获得 userId 相关的查询
        Set<Long> userIds = new HashSet<>();
        if (StrUtil.isNotEmpty(reqVO.getUserMobile())) {
            MemberUserRespDTO user = memberUserApi.getUserByMobile(reqVO.getUserMobile());
            if (user == null) { // 没查询到用户，说明肯定也没他的订单
                return null;
            }
            userIds.add(user.getId());
        }
        if (StrUtil.isNotEmpty(reqVO.getUserNickname())) {
            List<MemberUserRespDTO> users = memberUserApi.getUserListByNickname(reqVO.getUserNickname());
            if (CollUtil.isEmpty(users)) { // 没查询到用户，说明肯定也没他的订单
                return null;
            }
            userIds.addAll(convertSet(users, MemberUserRespDTO::getId));
        }
        return userIds;
    }

    @Override
    public TradeOrderSummaryRespVO getOrderSummary(TradeOrderPageReqVO reqVO) {
        // 根据用户查询条件构建用户编号列表
        Set<Long> userIds = buildQueryConditionUserIds(reqVO);
        if (userIds == null) { // 没查询到用户，说明肯定也没他的订单
            return new TradeOrderSummaryRespVO();
        }
        // 查询每个售后状态对应的数量、金额
        List<Map<String, Object>> list = tradeOrderMapper.selectOrderSummaryGroupByRefundStatus(reqVO, null);

        TradeOrderSummaryRespVO vo = new TradeOrderSummaryRespVO().setAfterSaleCount(0L).setAfterSalePrice(0L);
        for (Map<String, Object> map : list) {
            Long count = MapUtil.getLong(map, "count", 0L);
            Long price = MapUtil.getLong(map, "price", 0L);
            // 未退款的计入订单，部分退款、全部退款计入售后
            if (TradeOrderRefundStatusEnum.NONE.getStatus().equals(MapUtil.getInt(map, "refundStatus"))) {
                vo.setOrderCount(count).setOrderPayPrice(price);
            } else {
                vo.setAfterSaleCount(vo.getAfterSaleCount() + count).setAfterSalePrice(vo.getAfterSalePrice() + price);
            }
        }
        return vo;
    }

    @Override
    public PageResult<TradeOrderDO> getOrderPage(Long userId, AppTradeOrderPageReqVO reqVO) {
        return tradeOrderMapper.selectPage(reqVO, userId);
    }

    @Override
    public Long getOrderCount(Long userId, Integer status, Boolean commentStatus) {
        return tradeOrderMapper.selectCountByUserIdAndStatus(userId, status, commentStatus);
    }

    @Override
    public Long getOrderCount(Integer status, TradeOrderPageReqVO reqVO) {
        UserShopDetailsVO userShopDetails = userService.getUserShopDetails();

        return tradeOrderMapper.selectCountByStatus(status, userShopDetails.getShopId(), userShopDetails.getMerId(), reqVO);
    }

    @Override
    public Long getOrderCount(Integer status, Long shopId, Long merId) {
//        return tradeOrderMapper.selectCountByStatus(status, shopId, merId);
        return null;
    }

    @Override
    public List<PayOrderPageItemRespVO> getOrderNoAndPicUrl(List<PayOrderPageItemRespVO> list) {
        for (PayOrderPageItemRespVO payOrderPageItemRespVO : list) {
            TradeOrderDO tradeOrderDO = tradeOrderMapper.selectById(payOrderPageItemRespVO.getId());
            List<TradeOrderItemDO> tradeOrderItemDOS = tradeOrderItemMapper.selectListByOrderId(payOrderPageItemRespVO.getId());
            if (tradeOrderDO != null && !tradeOrderItemDOS.isEmpty()) {
                payOrderPageItemRespVO.setPicUrl(tradeOrderItemDOS.get(0).getPicUrl());
                payOrderPageItemRespVO.setOrderNo(tradeOrderDO.getNo());
            }
        }
        return list;
    }

    @Override
    public Integer getShopSalesVolume(Long key) {

        return tradeOrderMapper.selectByMerId(key);

    }

    @Override
    public List<TradeOrderDO> getOrderByMerId(Long id) {
        UserShopDetailsVO userShopDetails = userService.getUserShopDetails();

        return tradeOrderMapper.selectOrderListByMerId(id, userShopDetails.getMerId(), userShopDetails.getShopId());
    }

    @Override
    public TradeOrderDO getOrderById(Long orderId) {
        return tradeOrderMapper.selectOne(new LambdaQueryWrapper<TradeOrderDO>().eq(TradeOrderDO::getId, orderId).isNull(TradeOrderDO::getReceivingTime));
    }

    @Override
    public Map<String, Long> getBacklogOrder() {
        UserShopDetailsVO userShopDetails = userService.getUserShopDetails();
        Long backlogShipmentsCount = tradeOrderMapper.selectCountByStatus(TradeOrderStatusEnum.UNDELIVERED.getStatus(), userShopDetails.getShopId(), userShopDetails.getMerId());

        Map<String, Long> backMap = new HashMap<>();
        backMap.put(BacklogConstants.BACKLOG_SHIPMENTS, backlogShipmentsCount);
        return backMap;
    }

    @Override
    public Map<String, Long> getBacklogAfterSale() {
        UserShopDetailsVO userShopDetails = userService.getUserShopDetails();
        Long backlogAfterSaleCount = afterSaleService.selectCountByStatus(userShopDetails.getShopId(), userShopDetails.getMerId());

        Map<String, Long> backMap = new HashMap<>();
        backMap.put(BacklogConstants.BACKLOG_AFTER_SALE, backlogAfterSaleCount);
        return backMap;
    }

    @Override
    public Integer getSpuType(List<TradeOrderItemDO> orderItems) {
        for (TradeOrderItemDO orderItem : orderItems) {
            ProductSpuDO spu = productSpuService.getSpu(orderItem.getSpuId());
            if (spu.getSpuType() == 0) {
                return 0;
            }
        }
        return null;
    }

    @Override
    public Integer getSpuTypeByList(List<TradeOrderPageItemRespVO.Item> items) {
        for (TradeOrderPageItemRespVO.Item orderItem : items) {
            ProductSpuDO spu = productSpuService.getSpu(orderItem.getSpuId());
            if (spu.getSpuType() == 0) {
                return 0;
            }
        }
        return null;
    }

    @Override
    public Long getOrderCountByRefundStatus(Integer refundStatus, TradeOrderPageReqVO reqVO) {
        return tradeOrderMapper.selectCountByRefundStatus(refundStatus, reqVO);
    }

    @Override
    public EChartsResult getOrderCountWithMoney(LocalDate startTime, LocalDate endTime) {

        UserShopDetailsVO userShopDetails = adminUserService.getUserShopDetails();

        LocalDate today = LocalDate.now(); // 获取今天的日期

        if (startTime == null && endTime == null) {
            startTime = today.minusDays(6); // 获取今天的前七天
            endTime = today; // 获取今天
        }
        List<LocalDate> weekDates = generateWeekDates(startTime, endTime); // 生成本周的日期数组
        List<EChartsResult.Series> series = weekDates.stream().map(ignored -> new EChartsResult.Series().setCount(0).setMoney(0L)).collect(Collectors.toList());

        List<TradeOrderCountWithMoney> result = tradeOrderMapper.selectOrderCountWithTime(startTime, endTime, userShopDetails.getMerId(), userShopDetails.getShopId());
        for (int i = 0; i < weekDates.size(); i++) {
            for (TradeOrderCountWithMoney tradeOrderCountWithMoney : result) {
                if (weekDates.get(i).toString().equals(tradeOrderCountWithMoney.getDate())) {
                    series.get(i).setCount(tradeOrderCountWithMoney.getCount()).setMoney(tradeOrderCountWithMoney.getMoney());
                }
            }
        }
        return EChartsResult.builder()
                .xaxis(new EChartsResult.Axis()
                        .setData(weekDates))
                .series(series)
                .build()
                ;
    }

    /**
     * 生成指定日期范围内的日期数组
     *
     * @param start 开始日期
     * @param end   结束日期
     * @return 日期数组
     */
    private static List<LocalDate> generateWeekDates(LocalDate start, LocalDate end) {
        return Stream.iterate(start, date -> date.plusDays(1))
                .limit(end.toEpochDay() - start.toEpochDay() + 1)
                .collect(Collectors.toList());
    }

    @Override
    public List<ExpressTrackRespDTO> getExpressTrackList(Long id, Long userId) {
        // 查询订单
        TradeOrderDO order = tradeOrderMapper.selectByIdAndUserId(id, userId);
        if (order == null) {
            throw exception(ORDER_NOT_FOUND);
        }
        // 查询物流
        return getExpressTrackList(order);
    }

    @Override
    public List<ExpressTrackRespDTO> getExpressTrackList(Long id) {
        // 查询订单
        TradeOrderDO order = tradeOrderMapper.selectById(id);
        if (order == null) {
            throw exception(ORDER_NOT_FOUND);
        }
        // 查询物流
        return getExpressTrackList(order);
    }

    @Override
    public int getSeckillProductCount(Long userId, Long activityId) {
        // 获得订单列表
        List<TradeOrderDO> orders = tradeOrderMapper.selectListByUserIdAndSeckillActivityId(userId, activityId);
        orders.removeIf(order -> TradeOrderStatusEnum.isCanceled(order.getStatus())); // 过滤掉【已取消】的订单
        if (CollUtil.isEmpty(orders)) {
            return 0;
        }
        // 获得订单项列表
        return tradeOrderItemMapper.selectProductSumByOrderId(convertSet(orders, TradeOrderDO::getId));
    }

    /**
     * 获得订单的物流轨迹
     *
     * @param order 订单
     * @return 物流轨迹
     */
    private List<ExpressTrackRespDTO> getExpressTrackList(TradeOrderDO order) {
        if (order.getLogisticsId() == null) {
            return Collections.emptyList();
        }
        // 查询物流公司
        DeliveryExpressDO express = deliveryExpressService.getDeliveryExpress(order.getLogisticsId());
        if (express == null) {
            throw exception(EXPRESS_NOT_EXISTS);
        }
        // 查询物流轨迹
        return getSelf().getExpressTrackList(express.getCode(), order.getLogisticsNo(), order.getReceiverMobile(),order.getId());
    }

    /**
     * 查询物流轨迹
     * <p>
     * 缓存的目的：考虑及时性要求不高，但是每次调用需要钱
     *
     * @param code           快递公司编码
     * @param logisticsNo    发货快递单号
     * @param receiverMobile 收、寄件人的电话号码
     * @return 物流轨迹
     */
    @Cacheable(cacheNames = RedisKeyConstants.EXPRESS_TRACK, key = "#code + '-' + #logisticsNo + '-' + #receiverMobile", condition = "#result != null")
    public List<ExpressTrackRespDTO> getExpressTrackList(String code, String logisticsNo, String receiverMobile, Long orderId) {
        return expressClientFactory.getDefaultExpressClient().getExpressTrackList(new ExpressTrackQueryReqDTO().setExpressCode(code).setLogisticsNo(logisticsNo).setPhone(receiverMobile).setOrderId(orderId));
    }


    // =================== Order Item ===================

    @Override
    public TradeOrderItemDO getOrderItem(Long userId, Long itemId) {
        TradeOrderItemDO orderItem = tradeOrderItemMapper.selectById(itemId);
        if (orderItem != null && ObjectUtil.notEqual(orderItem.getUserId(), userId)) {
            return null;
        }
        return orderItem;
    }

    @Override
    public TradeOrderItemDO getOrderItem(Long id) {
        return tradeOrderItemMapper.selectById(id);
    }

    @Override
    public List<TradeOrderItemDO> getOrderItemListByOrderId(Collection<Long> orderIds) {
        if (CollUtil.isEmpty(orderIds)) {
            return Collections.emptyList();
        }

        return tradeOrderItemMapper.selectListByOrderId(orderIds);
    }


    /**
     * 获得自身的代理对象，解决 AOP 生效问题
     *
     * @return 自己
     */
    private TradeOrderQueryServiceImpl getSelf() {
        return SpringUtil.getBean(getClass());
    }

}
